(function (window, $){
	"use strict"
	var HAPYoutubeLoader = function (data){
		
		var self = this,
		settings = data,
		type,
		processData,
		finishData = [],
		deeplinkCounter,
		maxResults = 50,
		requestUrl,
		nextPageToken,
		limit,
		order,
		orderArr = ['date','rating','relevance','title','videoCount','viewCount'],//only valid for search?
		appId = data.youtubeAppId;

		this.resumeLoad = function(nextPageToken){
			finishData = [];

			if(nextPageToken){
				requestUrl = nextPageToken.substr(0,nextPageToken.lastIndexOf('&pageToken='));//remove last page token
				process(nextPageToken);
			}else{
				$(self).trigger('HAPYoutubeLoader.END_LOAD', {data:finishData, nextPageToken:nextPageToken});		
			}
		}
		this.setData = function(data){

			if(!appId){
				alert('Youtube API key missing! Set API key in settings.');
				return;
			}

			finishData = [];
			processData = data;
			deeplinkCounter = 0;
			limit = processData.limit || 400;
			maxResults = 50;
			if(limit < maxResults)maxResults = limit;
			order = processData.sort || 'relevance';
			type = processData.type;
			nextPageToken = null;

			//build snippet fields
			var s = '';
			if(settings.playlistItemContent.indexOf('title')>-1)s+='title,';
			if(settings.playlistItemContent.indexOf('description')>-1)s+='description,';
			if(settings.playlistItemContent.indexOf('date')>-1)s+='publishedAt,';
			if(settings.playlistItemContent.indexOf('thumb')>-1)s+='thumbnails,';
			if(s.length>0){
				s = s.substr(0,s.length-1)//remove comma
				var snippetTitle = ',snippet';
				var snippet = ',snippet('+s+')';
			}else{
				var snippetTitle = '';
				var snippet = '';
			}

			if(type == 'youtube_single' || type == 'youtube_single_list'){

				if(type == 'youtube_single_list')processData.path = processData.path.replace(/\s+/g, '');

				//build contentDetails fields
				if(settings.playlistItemContent.indexOf('duration')>-1){
					var contentDetailsTitle = ',contentDetails';
					var contentDetails = ',contentDetails(duration)';
				}else{
					var contentDetailsTitle = '';
					var contentDetails = '';
				}

				//video, https://developers.google.com/youtube/v3/docs/videos
				//requestUrl = 'https://www.googleapis.com/youtube/v3/videos?id='+processData.path+'&key='+appId+'&part=id'+snippetTitle+',status'+contentDetailsTitle+'&fields=items(id,status(privacyStatus)'+snippet+contentDetails+')';

				//without privacy status
				requestUrl = 'https://www.googleapis.com/youtube/v3/videos?id='+processData.path+'&key='+appId+'&part=id'+snippetTitle+contentDetailsTitle+'&fields=items(id'+snippet+contentDetails+')';

			}else{

				if(type == 'youtube_playlist'){

					//no duration in contentDetails here

					//https://developers.google.com/youtube/v3/docs/playlistItems/list
					//requestUrl = 'https://www.googleapis.com/youtube/v3/playlistItems?playlistId='+processData.path+'&maxResults='+maxResults+'&key='+appId+'&part=status'+snippetTitle+',contentDetails&fields=items(status(privacyStatus)'+snippet+',contentDetails(videoId))';

					//without privacy status
					requestUrl = 'https://www.googleapis.com/youtube/v3/playlistItems?playlistId='+processData.path+'&maxResults='+maxResults+'&key='+appId+'&part=contentDetails'+snippetTitle+'&fields=items(contentDetails(videoId)'+snippet+'),nextPageToken';

				}
			}

			process(requestUrl);
		
		}
		function process(url) {

			$.ajax({
				url: url,
				dataType: "jsonp"
			}).done(function(response) {
			
				if(response.error && response.error.message){
					console.log(response.error.message);
					return;
				} 

				var i, len = response.items.length, item;

				nextPageToken = response.nextPageToken;
				if(nextPageToken)nextPageToken = requestUrl+'&pageToken='+nextPageToken;//we have to save whole nextPageToken if we load from cache and then load more on total scroll

				for(i=0; i < len; i++){
					if(finishData.length == limit)break;

					item = response.items[i];

					if(type=='youtube_playlist' || type=='youtube_single' || type=='youtube_single_list'){
						
						if(item.status){
							if(item.status.privacyStatus == 'public')finishData.push(getItemData(item, type));
						}else if(item.snippet){
							if(item.snippet.title != 'Private video')finishData.push(getItemData(item, type));
						}
					}
				}

				if(type=='youtube_single' || type=='youtube_single_list'){

					$(self).trigger('HAPYoutubeLoader.END_LOAD', {data:finishData, nextPageToken:nextPageToken});		

			    }else{

					if(finishData.length < limit){

						if(nextPageToken){
							//adjust nextPageToken - when we request more than youtube can retrive (50) in one go, for example: 70 videos, in first request we will get 50 videos, then next page token will retrieve 50 videos again, but we only need next 20 videos! so we will loose leftover 30 videos from next page token. In this example we need to adjust maxResults to 20.

							if(finishData.length + maxResults > limit){
								maxResults = limit - finishData.length;
								var t1 = nextPageToken.substr(0,requestUrl.indexOf('&maxResults=')+12),
								t2 = nextPageToken.substr(requestUrl.indexOf('&key='));
								nextPageToken = t1+maxResults.toString()+t2;
							}

							if(type=='youtube_playlist'){
								var url = nextPageToken;
							}

							process(url);

						}else{

							$(self).trigger('HAPYoutubeLoader.END_LOAD', {data:finishData, nextPageToken:nextPageToken});	
							
						}
					}else{

						$(self).trigger('HAPYoutubeLoader.END_LOAD', {data:finishData, nextPageToken:nextPageToken});		
						
					}	
				}

			}).fail(function(jqXHR, textStatus, errorThrown) {
				console.log(jqXHR, textStatus, errorThrown);
			});	

		}
		function getItemData(item, type) {

			var obj = jQuery.extend(true, {}, processData);
			obj.type = 'youtube';

			if(type=='youtube_single' || type=='youtube_single_list'){
				obj.mp3 = item.id;
			}else if(type=='youtube_playlist'){
				obj.mp3 = item.contentDetails.videoId;
			}

			if(item.snippet){
				//obj.snippet = item.snippet;	
				if(!obj.title && item.snippet.title)obj.title = item.snippet.title;
				if(!obj.description && item.snippet.description)obj.description = item.snippet.description;
				if(item.snippet.publishedAt)obj.date = item.snippet.publishedAt;
				if(!item.thumb && item.snippet.thumbnails){
					/*
					default 120x90
					medium 320x180
					high 480x360
					standard 640x480
					maxres 1280x720
					*/
					if(item.snippet.thumbnails.medium)obj.thumb = item.snippet.thumbnails.medium.url;
					else if(item.snippet.thumbnails.high)obj.thumb = item.snippet.thumbnails.high.url;
					else if(item.snippet.thumbnails.default)obj.thumb = item.snippet.thumbnails.default.url;
				}
			}

			if(item.contentDetails){
				if(item.contentDetails.duration)obj.duration = ytDur2Sec(item.contentDetails.duration);
			} 
		
			return obj;
		}

		function ytDur2Sec(duration) {
		    var a = duration.match(/\d+/g);

		    if (duration.indexOf('M') >= 0 && duration.indexOf('H') == -1 && duration.indexOf('S') == -1) {
		        a = [0, a[0], 0];
		    }

		    if (duration.indexOf('H') >= 0 && duration.indexOf('M') == -1) {
		        a = [a[0], 0, a[1]];
		    }
		    if (duration.indexOf('H') >= 0 && duration.indexOf('M') == -1 && duration.indexOf('S') == -1) {
		        a = [a[0], 0, 0];
		    }

		    duration = 0;

		    if (a.length == 3) {
		        duration = duration + parseInt(a[0]) * 3600;
		        duration = duration + parseInt(a[1]) * 60;
		        duration = duration + parseInt(a[2]);
		    }

		    if (a.length == 2) {
		        duration = duration + parseInt(a[0]) * 60;
		        duration = duration + parseInt(a[1]);
		    }

		    if (a.length == 1) {
		        duration = duration + parseInt(a[0]);
		    }
		    return duration;
		}

		this.getNextPageToken = function(){
			return nextPageToken;
		}
	
	};	

	window.HAPYoutubeLoader = HAPYoutubeLoader;

}(window,jQuery));
